*----------------- BEGIN V2AD -----------------------------
      subroutine v2ad(vel, npts, dt, acc, dis)


* Compute acceleration and displacement from velocity, using the reverse process
* of going from acceleration to velocity,
* assuming that the acceleration
* is represented by straight lines connecting the digitized values.

* For consistency, I assume the vel(1) and acc(1) = 0.0, even if that is not
* true of vel(1).

* But note this comment in acc2vd.for:
*        01/07/00 - Added initial velocity and displacements (v0, d0).
*                   This also requires the addition of a linear trend in
*                   displacement.
*                   Also, Bill Joyner, Chris Stephens, and I considered how to
*                   handle the first point.  Assuming v0 = 0, BAP uses a 
*                   trapezoidal rule such that the first vel(1) = 0.5*dt*a(1),
*                   but in acc2vd, vel(1) = 0.  In effect, BAP starts the
*                   integration at -dt rather than 0.0, with the result that
*                   because vel(1) depends on dt, vel(1) will change if the
*                   digitization interval is changed.  We agreed that this is 
*                   not correct.

* Dates: 06/28/04 - Written by D.M. Boore, patterned after 
*                   v2a.for and acc2vd.for.  Assumes a0 = d0 = 0.0
*        06/28/04 - Use simple difference for accel
*        06/28/04 - Use centered difference to avoid the time shift produced
*                   by a one-sided difference
*        06/29/04 - Back to one-sided difference
*        06/30/04 - Slight revision in computing acc(1) (no longer 
*                   assume v1 = 0.0)
*        04/29/09 - Now use a five-point centered difference, assuming
*                   that vel = 0 outside the input motion.

      real acc(*), vel(*), dis(*)
      double precision vm2, vm1, vp1, vp2,
     :                 f1, f2

      double precision v1, v2, a1, a2, cumd,
     : ddt, ddt_2, ddtdt_6

      ddt     = dble(dt)
      ddt_2   = 0.5d0 * dble(dt)
      ddtdt_6 = dble(dt)*dble(dt)/6.0d0

* compute acceleration and displacement, using analytical formulas based
* on representing the acceleration as a series of straightline segments.

!      acc(1) = a0
      acc(1) = 0.0 

      do i = 1, npts
        acc(i) = 0.0
      end do

! Do the following so do not actually alter input array vel
      cumd = 0.0d0
      d0 = 0.0
      dis(1) = sngl(cumd) + d0

*      v1 = 0.0d0
      v1 = dble(vel(1))
      v2 = dble(vel(2))

      a1 = 0.0d0
      a2 = (v2-v1)/ddt
*      a2 = (v2-v1)/ddt_2 - a1
!      acc(2) = sngl(a2)

      cumd = cumd + v1*ddt + (2.0*a1 + a2)*ddtdt_6
      dis(2) = sngl(cumd) + d0  ! linear term accounted for in v1*ddt

      do j=3,npts
        v1 = dble(vel(j-1))
        v2 = dble(vel(j))
        a1 = a2
        a2 = (v2-v1)/ddt
*        a2 = (v2-v1)/ddt_2 - a1
!        acc(j) = sngl(a2)
        cumd = cumd + v1*ddt + (2.0*a1 + a2)*ddtdt_6
        dis(j) = sngl(cumd) + d0  ! linear term accounted for in v1*ddt
      end do

* Now redo acceleration using centered differences:

*      do j=3,npts-1
*        vm1 = dble(vel(j-1))
*        vp1 = dble(vel(j+1))
*        acc(j) = sngl(0.5d0*(vp1-vm1)/ddt)
*      end do

        f1=1.0d0/12.0d0
        f2=2.0d0/3.0d0
        
        do i=3,npts-2
          vm2 = dble(vel(i-2))
          vm1 = dble(vel(i-1))
          vp1 = dble(vel(i+1))
          vp2 = dble(vel(i+2))
          acc(i)=sngl((f1*vm2-f2*vm1+f2*vp1-f1*vp2)/ddt)
        end do

        i = 1
        vm2 = 0.0d0
        vm1 = 0.0d0
        vp1 = dble(vel(i+1))
        vp2 = dble(vel(i+2))
        acc(1)= sngl((f1*vm2-f2*vm1+f2*vp1-f1*vp2)/ddt)
        i = 2
        vm2 = 0.0d0
        vm1 = dble(vel(i-1))
        vp1 = dble(vel(i+1))
        vp2 = dble(vel(i+2))
        acc(i)= sngl((f1*vm2-f2*vm1+f2*vp1-f1*vp2)/ddt)
        i = npts-1
        vm2 = dble(vel(i-2))
        vm1 = dble(vel(i-1))
        vp1 = dble(vel(i+1))
        vp2 = 0.0d0
        acc(i)= sngl((f1*vm2-f2*vm1+f2*vp1-f1*vp2)/ddt)
        i = npts
        vm2 = dble(vel(i-2))
        vm1 = dble(vel(i-1))
        vp1 = 0.0d0
        vp2 = 0.0d0
        acc(i)= sngl((f1*vm2-f2*vm1+f2*vp1-f1*vp2)/ddt)
      
      return
      end
*----------------- END V2AD -----------------------------

